/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file btengine.cc
/// \brief Implementation of Btengine class

#include "btengine.hh"

#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/rfcomm.h>


//////////////////////////////////////
//				PUBLIC				//
//////////////////////////////////////


/// \brief Constructor
Btengine::Btengine( std::string bda, unsigned int port ) 
	: bda_( bda ), port_( port )
{
}

// Destructor
Btengine::~Btengine()
{
}

// Connect to remote server
bool Btengine::connectServer()
{	
	if( bda_ == "" || bda_.length()  != 17 )
	{
		std::cout << "Invalid bda: " << bda_ << ", length: " 
		<< bda_.length() << std::endl;
		
		return false;
	}
	
	// create the socket
	int connection = 0;
	if( ( connection = socket( AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM ) )
		== -1 )
	{
		std::cout << "BT connection failed at socket" << std::endl;
		return false;
	}
	
	// in order to create rfcomm connection, we must fill in
	// sockaddr_rc structure
	// struct sockaddr_rc {
	// sa_family_t	rc_family;
	// bdaddr_t			rc_bdaddr;
	// uint8_t			rc_channel;
	// };
	struct sockaddr_rc address;
	address.rc_family = AF_BLUETOOTH;
	address.rc_channel = 1;
	str2ba( bda_.c_str() , &address.rc_bdaddr );
	
	// try connecting to server
	if( connect( connection, ( struct sockaddr* ) &address, sizeof( address ) ) 
		== -1 )
	{
		std::cout << "BT connection failed at connect" << std::endl;
		Engine::disconnect();
		return false;
	}
	
	// connection established
	Engine::setConnection( connection );
	Engine::setConnected( true );
	
	return true;
}
