/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// file btengine.hh
/// \brief Declaration of Btengine class

#ifndef BTENGINE_HH
#define BTENGINE_HH

#include "engine.hh"

#include <cstdlib>
#include <string>

/// \brief Class for connection via bluetooth
///
/// This class provides connection to mythetomerserver via bluetooth. It can
/// be used to send messages to the server. All communication happends via 
/// bluetooth.
class Btengine: public Engine {
	public:
		
		/// \brief Constructor
		///
		/// Parameters: bda - Bluetooth Device Address, the address of the
		///					  bluetooth device to be connected
		///				port - port of the remove server at given BDA
		Btengine( std::string bda, unsigned int port );
		
		/// \brief Destructor
		virtual ~Btengine();
		
		/// \brief Connect to remote server
		///
		/// Use this function to establish a bluetooth connection to the 
		/// given BDA and port.
		///
		/// Return value: true - connection successfully established
		///				  false - failed to establish connection
		virtual bool connectServer();
		
	private:
		
		/// Bluetooth Device Address of the mythetomerserver
		std::string bda_;
		
		/// port of the mythetomerserver service
		unsigned int port_;
};

#endif
