/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file engine.cc
/// \brief Implementation of Engine class

#include "engine.hh"

#include <iostream>


//////////////////////////////
// 			PUBLIC 			//
//////////////////////////////


// Constructor
Engine::Engine() : connection_( 0 ), connected_( false )
{
}

// Virtual destructor
Engine::~Engine()
{
	if( connected_ )
	{
		disconnect();
	}
}

// Disconnect from remote server
void Engine::disconnect()
{
	// close the connection
	close( connection_ );
	
	// mark disconnected
	connected_ = false;
}

// Get status whether the connection is currently established
bool Engine::isConnected() const
{
	return connected_;
}

// Send message to remote server
bool Engine::sendMsg( std::string message )
{
	// write message to socket and check the result
	if( write( connection_, message.c_str(), message.length() ) == -1 )
	{
		std::cout << "Command sending failed at write" << std::endl;
		
		// sending failed, disconnect
		disconnect();
		return false;
	}
	
	// the message was successfully sent
	return true;
}


//////////////////////////////
// 			PROTECTED 		//
//////////////////////////////


// Set connection socket
void Engine::setConnection( int connection )
{
	connection_ = connection;
}

// Set connection status
void Engine::setConnected( bool status )
{
	connected_ = status;
}
