/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file ipengine.cc
/// \brief Implementation of Ipengine class

#include "ipengine.hh"
#include "routines.hh"

#include <iostream>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h> 
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <unistd.h>


//////////////////////////////
// 			PUBLIC 			//
//////////////////////////////


// Constructor
Ipengine::Ipengine( std::string serverAddress, unsigned int port )
	: server_( serverAddress ), port_( port )
{
}

// Destructor
Ipengine::~Ipengine()
{
}

// Connect to remote server
bool Ipengine::connectServer()
{
	// look up for host information
	if( server_ == "" )
	{
		std::cout << "Invalid remote ip address" << std::endl;
		return false;
	}
	
	hostent* host = 0;
	host = gethostbyname( server_.c_str() );
	
	if( host == NULL )
	{
		std::cout << "Error looking up for hostname" << std::endl;
		return false;
	}
	
	// create socket
	int connection = 0;
	if( ( connection = socket( PF_INET, SOCK_STREAM, 0 ) ) == -1 )
	{
		std::cout << "IP connection failed at socket" << std::endl;
		return false;
	}
	
	// in order to create ip connection, we must fill in
	// struct sockaddr_in {
   //  short   sin_family; 	// address family
   //  u_short sin_port;		// port number
   //  struct  in_addr sin_addr; // internet address
   //  char    sin_zero[8]; 	// same size as struct sockaddr
	// };
	struct sockaddr_in address;
	memset( &address, 0, sizeof( address ) );
	address.sin_family = AF_INET;
	address.sin_port = htons( port_ );
	address.sin_addr = *( ( struct in_addr* ) host->h_addr );
	
	// try connecting the server
	if( connect( connection, ( struct sockaddr* ) &address, sizeof( address ) )
		== -1 )
	{
		std::cout << "IP connection failed at connect" << std::endl;
		Engine::disconnect();
		return false;
	}
	
	// connection established
	Engine::setConnection( connection );
	Engine::setConnected( true );
	
	return true;
}
