/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file mythetomer.cc
/// MythTV bluetooth/ip remote control for Maemo Platform

// to compile for OS2008, run
// g++ -o mythetomer mythetomer.cc ui.cc engine.cc btengine.cc ipengine.cc settings.cc connection.cc routines.cc -lbluetooth `pkg-config hildonmm hildon-fmmm hildon-1 libosso hildon-help conic gconf-2.0 --cflags --libs` -ansi -Wall

#include "ui.hh"
#include "settings.hh"
#include "engine.hh"
#include "btengine.hh"
#include "ipengine.hh"
#include "routines.hh"
#include "connection.hh"

#include <gtkmm/main.h>
#include <hildonmm/init.h>
#include <iostream>
#include <sstream>
#include <fstream>

/// \brief Connect to remote server
///
/// This function is used for connecting automatically to server when the 
/// processor has idle time after startup. It just tries to connect the remote
/// server. This function must not be used elsewhere since it doesn't check
/// return value (succeed/failure) from the engine.
///
/// Parameters: none
///
/// Return value: true - function stays in idle-queue and gets called again
///				  false  - function is removed from idle-queue
bool autoConnect( Ui* gui );

int main( int argc, char* argv[] )
{
	// init gktmm
	Gtk::Main gtkmm( &argc, &argv );
	// init hildon
	Hildon::init();
	
	// init osso
	osso_context_t* ossoContext = osso_initialize( "mythetomer", "0.3", TRUE, NULL );
	
	// check the osso init result
	if( ossoContext == NULL )
	{
		std::cout << "osso_initialize() failed" << std::endl;
		return OSSO_ERROR;
	}
	
	// set application name
	Glib::set_application_name( "Mythetomer" );
	
	// acquire system dbus
	DBusError error;
	dbus_error_init( &error );
	
	// get the system bus
	DBusConnection* dbus = dbus_bus_get( DBUS_BUS_SYSTEM, &error );
	
	if( !dbus )
	{
		std::cout << "Failed to open connection to system bus: " 
		<< error.message << std::endl;
		dbus_error_init( &error );
		return EXIT_FAILURE;
	}
	
	// create and load settings
	Settings* settings = new Settings();
	
	// check that loading of settings succeeded
	if( settings != NULL && !settings->loadSettings() )
	{
		std::cout << "Could not load settings" << std::endl;
	}
	
	Connection* connection = new Connection( settings );
	
	// create user interface and set it to program
	Ui gui( ossoContext, dbus, settings, connection ); 
	Hildon::Program::get_instance()->add_window( gui );
	
	// check if autoconnect was preferred
	if( settings->getAutoconnectStatus() )
	{
		// start connecting server after gui is loaded
		g_idle_add( (GSourceFunc) autoConnect, &gui );
	}
	
	// show the gui and run until it is closed
	gtkmm.run( gui );
	
	// the code below is run when exiting
	
	// save settings to file
	settings->saveSettings();
	
	// release memory and clear pointers
	delete connection;
	connection = 0;
	
	delete settings;
	settings = 0;
	
	// release dbus
	dbus_connection_unref( dbus );
	dbus = NULL;
	
	// deinit osso
	osso_deinitialize( ossoContext );
	
}

// Connect to remote server
bool autoConnect( Ui* gui )
{	
	// start the default server server
	if( gui != NULL )
	{
		//engine->connectServer();
		gui->openConnection();
	}
	
	// return false so that function is removed from idle-queue
	return false;
}
