/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file routines.cc
/// \brief Implementation of common functions,routines, and callback funtions
/// collected together into a namespace.

#include "routines.hh"
#include "ui.hh"
#include "connection.hh"

#include <sstream>
#include <iostream>
#include <conbtdialogs-dbus.h>

namespace Routines
{
	// Converts integer to string
	std::string int2string( int number )
	{
		std::stringstream stream;
		stream << number;
		return stream.str();
	}
	
	
	// Converts string to int
	bool string2int( std::string const& string, unsigned int& number )
	{
		// make a stream
		std::istringstream stream( string ); 
		
		int i = 0;
		std::string trash = "";
		
		// if reading succeeded and there was no trailing trash
		if( stream >> i && !( stream >> trash ) && i >= 0 )
		{
			number = i; 
			return true; 
		}
	
	// converting didn't succeed
	return false; 
	}
	
	
	// Callback function for Bluetooth device search
	DBusHandlerResult btSearchHandler( DBusConnection* connection,
		DBusMessage* message, void* receiver )
	{
		// signal for bt search dialog
		if( dbus_message_is_signal( message, CONBTDIALOGS_DBUS_INTERFACE, 
			CONBTDIALOGS_SEARCH_SIG ) )
		{
			DBusError error;
			dbus_error_init( &error );
			char* address = NULL;
			char* device = NULL;
			
			if( !dbus_message_get_args( message, &error, DBUS_TYPE_STRING, &address,
				DBUS_TYPE_STRING, &device, DBUS_TYPE_INVALID ) )
			{
				std::cout << "Error getting dbus msg args" << std::endl;
				return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
			}
			
			std::string bda( address );
			std::string name( device );
			
			if( bda == "" )
			{
				std::cout << "Cancel was pressed" << std::endl;
			}
			else
			{	
				if( receiver != NULL )
				{
					Ui* gui = ((Ui*) (receiver));
					gui->setDevice( name, bda );
				}
				
			}
			
			return DBUS_HANDLER_RESULT_HANDLED;
		}
	
		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	}
	
	
	// Callback function for WLAN connection opening / status changes
	void ipConnectionEvent( ConIcConnection* connection, 
		ConIcConnectionEvent* event, gpointer caller )
	{
		Connection* conn = ((Connection*) (caller));
		
		// let connection class handle the event
		if( conn != NULL )
		{
			conn->handleConnectionEvent( event );
		}
		return;
	}
	
	
	// Print connection profile information
	void printProfile( Profile* profile )
	{
		std::cout << "Name: " << profile->name << std::endl;
		std::cout << "WLAN: " << profile->wlan << std::endl;
		std::cout << "IP: " << profile->ip << std::endl;
		std::cout << "BT: " << profile->bluetooth << std::endl;
		std::cout << "BDA: " << profile->bda << std::endl;
		std::cout << "Port: " << profile->port << std::endl;
		
	}

}
