/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file routines.hh
/// \brief Declaration of common functions,routines, and callback funtions
/// collected together into a namespace.

#ifndef ROUTINES_HH
#define ROUTINES_HH

#include "settings.hh"

#include <libosso.h>

#ifndef OS2007
	#include <conic.h>
#else
	#include <osso-ic-ui-dbus.h>
#endif

#include <string>

namespace Routines
{
	/// \brief Converts integer to string
	///
	/// Use this function to convert integer to string
	///
	/// Return value: the converted string
	///
	/// Parameters: number - the integer you want to be converted to string
	std::string int2string( int number );
	
	/// \brief Converts string to int
	///
	/// Use this function to convert string to integer
	///
	/// Return value: true - if the convert was successfull
	///				  false - convertion failed
	///
	/// Parameters: string - the string you want to convert
	///				 number - the resulting integer
	bool string2int( std::string const& string, unsigned int& number );
	
	/// \brief Callback function for Bluetooth device search
	///
	/// This function is used as a callback function for Bluetooth device
	/// search.
	///
	/// Return value: whether the event was handled or not
	///
	/// Parameters: receiver - object that should receive the result
	DBusHandlerResult btSearchHandler( DBusConnection* connection,
			DBusMessage* message, void* receiver );
	
	/// \brief Callback function for WLAN connection opening / status changes
	///
	/// This function is used as a callback function for opening WLAN
	/// connection and to monitor changes in connection status.
	///
	/// Return value: none
	///
	/// Parameters: caller - object that should receive the result
	void ipConnectionEvent( ConIcConnection* connection, 
		ConIcConnectionEvent* event, gpointer caller );
	
	/// \brief Print connection profile information
	///
	/// Use this function to print connection profile information to default
	/// output.
	///
	/// Return value: none
	///
	/// Parameters: profile - profile to be printed
	void printProfile( Profile* profile );
	
}

#endif
