/* Mythetomer Internet Tablet Client: a remote controller for MythTV
 * Copyright (C) 2007-2008 Riku Pelander
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \file ui.cc
/// \brief Implementation of UI class

#include "ui.hh"
#include "routines.hh"

#include <iostream>
#include <sstream>
#include <fstream>
#include <conbtdialogs-dbus.h>
#include <gdk/gdkkeysyms.h>
#include <hildonmm/number-editor.h>
#include <hildonmm/banner.h>

#ifndef OS2007
	#include <hildon/hildon-help.h>
#else
	#include <osso-helplib.h>
#endif

int const SMALL_BUTTON_SIZE = 85;
int const NUMBER_BUTTON_WIDTH = 70;
int const NUMBER_BUTTON_HEIGHT = -1;

// actions for buttons
std::string const ACTION_UP = "032";
std::string const ACTION_ENTER = "031";
std::string const ACTION_DOWN = "034";
std::string const ACTION_VOL_UP = "036";
std::string const ACTION_ONE = "0411";
std::string const ACTION_TWO = "0412";
std::string const ACTION_THREE = "0413";
std::string const ACTION_CH_UP = "038";
std::string const ACTION_FOUR = "0414";
std::string const ACTION_FIVE = "0415";
std::string const ACTION_SIX = "0416";
std::string const ACTION_VOL_DOWN = "037";
std::string const ACTION_SEVEN = "0417";
std::string const ACTION_EIGHT = "0418";
std::string const ACTION_NINE = "0419";
std::string const ACTION_CH_DOWN = "039";
std::string const ACTION_MENU = "0429";
std::string const ACTION_ZERO = "0410";
std::string const ACTION_ESC = "0430";
std::string const ACTION_REWIND = "0425";
std::string const ACTION_PAUSE = "0421";
std::string const ACTION_FORWARD = "0426";
std::string const ACTION_PREVIOUS = "0427";
std::string const ACTION_RECORD = "0422";
std::string const ACTION_NEXT = "0428";
std::string const ACTION_RIGHT = "033";
std::string const ACTION_LEFT = "035";
// actions for menu items
std::string const ACTION_MAIN_MENU = "0432";
std::string const ACTION_PROGRAM_GUIDE = "0433";
std::string const ACTION_LIVE_TV = "0434";
std::string const ACTION_TV_RECORDINGS = "0435";
std::string const ACTION_MYTHVIDEO = "0436";
std::string const ACTION_PLAY_DVD = "0437";
std::string const ACTION_PLAY_MUSIC = "0438";
std::string const ACTION_ASPECT_RATIO = "0439";
std::string const ACTION_TIME_STRETCH = "0440";
std::string const ACTION_INPUT_CARD = "0441";
std::string const ACTION_SLEEP_TIMER = "0442";
// strings related to help framework
std::string const OSSO_HELP_ABOUT = "help_mythetomer_about";
std::string const OSSO_HELP_INTRO = "help_mythetomer_introduction";


//////////////////////////////////////
//				PUBLIC				//
//////////////////////////////////////


// Constructor
Ui::Ui( osso_context_t* ossoContext, DBusConnection* dbus, 
	Settings* settings, Connection* connection ) :
	osso_( ossoContext ), dbus_( dbus ), settings_( settings ), 
	connection_( connection ), fullscreen_( 0 ), buildingSubmenu_( false ),
	vBoxLeft_( false, 3 ), imgUp_( Gtk::Stock::GO_UP, Gtk::ICON_SIZE_BUTTON ),
	imgEnter_( Gtk::Stock::OK, Gtk::ICON_SIZE_BUTTON ),
	imgDown_( Gtk::Stock::GO_DOWN, Gtk::ICON_SIZE_BUTTON ),
	vBoxCenter_( true, 3 ), hBox1_( true, 5 ), 
	imgVolUp_( Gtk::Stock::ADD, Gtk::ICON_SIZE_BUTTON ), btnOne_( "1" ),
	btnTwo_( "2" ), btnThree_( "3" ), 
	imgChUp_( Gtk::Stock::ADD, Gtk::ICON_SIZE_BUTTON ), hBox2_( true, 5 ),
	lblVol_( "Volume" ), btnFour_( "4" ), btnFive_( "5" ), btnSix_( "6" ),
	lblCh_( "Channel" ), hBox3_( true, 5 ), 
	imgVolDown_( Gtk::Stock::REMOVE, Gtk::ICON_SIZE_BUTTON ), 
	btnSeven_( "7" ), btnEight_( "8" ),
	btnNine_( "9" ), imgChDown_( Gtk::Stock::REMOVE, Gtk::ICON_SIZE_BUTTON ),
	hBox4_( true, 5 ), btnMenu_( "Menu" ), lblEmpty1_( "" ), btnZero_( "0" ),
	lblEmpty2_( "" ), btnEsc_( "Esc" ), hBox5_( true, 5 ),
	imgRewind_( Gtk::Stock::MEDIA_REWIND, Gtk::ICON_SIZE_BUTTON ),
	imgPause_( Gtk::Stock::MEDIA_PAUSE, Gtk::ICON_SIZE_BUTTON ),
	imgForward_( Gtk::Stock::MEDIA_FORWARD, Gtk::ICON_SIZE_BUTTON ),
	hBox6_( true, 5 ),
	imgPrevious_( Gtk::Stock::MEDIA_PREVIOUS, Gtk::ICON_SIZE_BUTTON ),
	imgRecord_( Gtk::Stock::MEDIA_RECORD, Gtk::ICON_SIZE_BUTTON ),
	imgNext_( Gtk::Stock::MEDIA_NEXT, Gtk::ICON_SIZE_BUTTON ), 
	vBoxRight_( false, 3 ),
	imgRight_( Gtk::Stock::GO_FORWARD, Gtk::ICON_SIZE_BUTTON ),
	imgEnterRight_( Gtk::Stock::OK, Gtk::ICON_SIZE_BUTTON ),
	imgLeft_( Gtk::Stock::GO_BACK, Gtk::ICON_SIZE_BUTTON ),
	itemGoto_( "Go to" ),
	itemMainMenu_( "Main menu" ), itemProgramGuide_( "Program Guide" ),
	itemLiveTV_( "Live TV" ), itemTVRecordings_( "TV Recordings" ),
	itemMythVideo_( "MythVideo" ), itemPlayDVD_( "Play DVD" ), 
	itemPlayMusic_( "Play music" ), itemExtraCommands_( "Extra commands" ),
	itemAspectRatio_( "Aspect ratio" ), itemTimeStretch_( "Time stretch" ),
	itemInputCard_( "Input card" ), itemSleepTimer_( "Sleep timer" ),
	itemCustomCommand_( "Custom command" ), systemCommandsSubmenu_( NULL ),
	itemSystemCommands_( "System commands" ), selectProfileSubmenu_( NULL ),
	itemSelectProfile_( "Connection profile" ), 
	itemOpenConnection_( "Open connection" ),
	itemCloseConnection_( "Close connection" ),
	itemTools_( "Tools" ), itemSettings_( "Settings" ), itemHelp_( "Help" ),
	itemAbout_( "About" ), itemClose_( "Close" )
{
	
	// connect hardware button listener
	signal_key_press_event().connect( 
		sigc::mem_fun( *this, &Ui::hwKeyPressed ), false );
		
	// check if need to start full screen
	if( settings->getFullscreenStatus() )
	{
		toggleFullscreen();
	}
	
	// build application menu
	buildMenu();
	
	// build application interface
	buildInterface();
	
	// make all child widgets visible
	show_all_children();
	
}

// Destructor
Ui::~Ui()
{
	// clear system commands menu items
	clearSystemCommands();
	
	// clear dynamically created menu items
	
	if( systemCommandsSubmenu_ != NULL )
	{
		delete systemCommandsSubmenu_;
	}
	
	// clear select profile menu items
	clearSelectProfileMenuItems();
	
	if( selectProfileSubmenu_ != NULL )
	{
		delete selectProfileSubmenu_;
	}
}

// Show information banner to user
void Ui::showMessage( std::string const& message )
{
	// show infoprint (information banner) on screen statusbar
	Hildon::Banner::show_information( *this, message );
	//osso_system_note_infoprint( osso_, message.c_str(), NULL );
}

// Callback function to open connection
void Ui::openConnection()
{
	// try to open connection
	Status status = connection_->openConnection();
	
	// check connection status and report to user
	switch( status )
	{
		// connection was already online
		case WASCONNECTED:
		{
			showMessage( "Connection already open" );
			break;
		}
		// connection successfully established
		case CONNECTED:
		{
			Profile profile = {};
			if( settings_ != NULL && settings_->getActiveProfile( profile ) )
			{
				showMessage( "Connection established to " + profile.name );
			}
			else
			{
				showMessage( "Connection established" );
			}
			break;
		}
		// connection was disconnected
		case DISCONNECTED:
		{
			break;
		}
		// connection procedure failed
		case FAILED:
		{
			Profile profile = {};
			if( settings_ != NULL && settings_->getActiveProfile( profile ) )
			{
				showMessage( "Could not establish connection to " 
					+ profile.name );
			}
			else
			{
				showMessage( "Could not establish connection" );
			}
			break;
		}
		default:
		{
			break;
		}
	}
}

// Callback function for inserting data after bluetooth search
void Ui::setDevice( std::string const & name, std::string const& bda )
{	
	if( ntrBda_ != NULL )
	{
		ntrBda_->set_text( bda );
	} 
}


//////////////////////////////////////
//				PRIVATE				//
//////////////////////////////////////


// Send user initiated command to the server
void Ui::userAction( std::string const& message )
{
	if( connection_ != NULL )
	{
		// try sending the command
		if( !connection_->sendMsg( message ) )
		{
			std::string msg = "Mythetomer is not connected. Do you want to establish a connection";
			
			Profile profile = {};
			if( settings_ != NULL && settings_->getActiveProfile( profile ) )
			{
				msg += " to ";
				msg += profile.name;
			}
			msg += "?";
			
			// create and show note
			Hildon::Note note( Hildon::NOTE_TYPE_CONFIRMATION_BUTTON, msg );
			note.add_button( Gtk::Stock::YES, Gtk::RESPONSE_YES );
			note.add_button( Gtk::Stock::NO, Gtk::RESPONSE_NO );
			
			int result = note.run();
			
			switch( result )
			{
				case Gtk::RESPONSE_YES:
				{
					// user requested connection
					openConnection();
					break;
				}
				case Gtk::RESPONSE_NO:
				default:
				{
					return;	
				}
			}
		}
	}
}

// Show dialog to send custom command
void Ui::showCustomCommandDialog()
{
	Gtk::Dialog dlgCustomCommand( "Send custom command", true, false );
	
	// add buttons to the dialog
	dlgCustomCommand.add_button( Gtk::Stock::OK, Gtk::RESPONSE_OK );
	dlgCustomCommand.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
	
	// create text field
	Gtk::Entry ntrCommand;
	dlgCustomCommand.get_vbox()->pack_start( ntrCommand );
	ntrCommand.show();
	
	// show the dialog
	int result = dlgCustomCommand.run();
	
	switch( result )
	{
		case Gtk::RESPONSE_OK:
		{
			// get command
			std::string commandString = ntrCommand.get_text();
			
			// count length
			std::string length = Routines::int2string( 
				commandString.length() + 4 );
				
			// formulate command according to connection spec
			if( commandString.length() < 6 )
			{
				commandString = "0" + length + "90" + commandString;
			}
			else
			{
				commandString = length + "90" + commandString;
			}
			
			// send command to server
			userAction( commandString );
			break;
		}
		case Gtk::RESPONSE_CANCEL:
		case Gtk::RESPONSE_DELETE_EVENT:
		{
			break;
		}
		default:
		{
			break;
		}
	}
}

// Show dialog to edit application settings
void Ui::showSettingsDialog()
{
	Gtk::Dialog dlgSettings( "Settings", true, false );
	
	// add buttons to the dialog
	dlgSettings.add_button( Gtk::Stock::OK, Gtk::RESPONSE_ACCEPT );
	dlgSettings.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
	
	// attach notebook
	dlgSettings.get_vbox()->pack_start( ntbkSettings_ );
	
	// build General tab
	Gtk::Table tblGeneral( 1, 2, true );
	ntbkSettings_.append_page( tblGeneral, "General" );
	
	// frame for startup options
	Gtk::Frame frmStartup( "Startup options");
	tblGeneral.attach( frmStartup, 0, 1, 0, 1, Gtk::FILL, Gtk::FILL, 2, 4 );
	Gtk::VBox vbxStartup;
	frmStartup.add( vbxStartup );
	
	// start fullscreen option
	Gtk::CheckButton btnFullscreen( "Start fullscreen" );
	vbxStartup.pack_start( btnFullscreen );
	
	if( settings_->getFullscreenStatus() )
	{
		btnFullscreen.set_active();
	}
	
	// connect automatically option
	Gtk::CheckButton btnAutoConnect( "Connect automatically" );
	vbxStartup.pack_start( btnAutoConnect );
	
	if( settings_->getAutoconnectStatus() )
	{
		btnAutoConnect.set_active();
	}
	
	// frame for profile at startup
	Gtk::Frame frmProfile( "Profile at startup" );
	tblGeneral.attach( frmProfile, 1, 2, 0, 1, Gtk::FILL, Gtk::FILL, 2, 4 );
	Gtk::VBox vbxProfile;
	frmProfile.add( vbxProfile );
	
	// group for radio button
	Gtk::RadioButtonGroup group;
	
	// default profile option
	Gtk::RadioButton btnDefaultProfile( group, "default profile" );
	vbxProfile.pack_start( btnDefaultProfile );
	
	// previously used option
	Gtk::RadioButton btnLastProfile( group, "previously used" );
	vbxProfile.pack_start( btnLastProfile );
	
	// put the right radio button active
	if( settings_->getUseDefaultProfile() )
	{
		btnDefaultProfile.set_active();
	}
	else
	{
		btnLastProfile.set_active();
	}
	
	// build Profiles tab
	Gtk::Table tblProfiles( 2, 4, false );
	ntbkSettings_.append_page( tblProfiles, "Profiles" );
	
	// combo box for profile names
	tblProfiles.attach( cmbProfiles_, 0, 4, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	
	// fill in the combo
	buildProfilesCombo();
	
	// select the active profile to be active in the list
	Profile profile = {};
	if( settings_->getActiveProfile( profile ) )
	{
		cmbProfiles_.set_active_text( profile.name );
	}
	
	// create new -button
	Gtk::Button btnCreateNew( "Create new" );
	tblProfiles.attach( btnCreateNew, 0, 1, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	btnCreateNew.signal_clicked().connect( 
		sigc::mem_fun( *this, &Ui::showNewProfileDialog ) );
	
	// edit button
	Gtk::Button btnEdit( "Edit" );
	tblProfiles.attach( btnEdit, 1, 2, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	btnEdit.signal_clicked().connect(
		sigc::mem_fun( *this, &Ui::showEditProfileDialog ) );
	
	// set as default -button
	Gtk::Button btnSetDefault( "Set as default" );
	tblProfiles.attach( btnSetDefault, 2, 3, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	btnSetDefault.signal_clicked().connect( 
		sigc::mem_fun( *this, &Ui::setDefaultProfile ) );
		
	// delete button
	Gtk::Button btnDelete( "Delete" );
	tblProfiles.attach( btnDelete, 3, 4, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	btnDelete.signal_clicked().connect( 
		sigc::mem_fun( *this, &Ui::deleteProfile ) ); 
	
	
	// build System commands tab
	Gtk::Table tblSystemCommands( 2, 2, false );
	ntbkSettings_.append_page( tblSystemCommands, "System commands" );
		
	// combo box for profile names
	tblSystemCommands.attach( cmbSystemCommands_, 0, 4, 0, 1, 
		Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	
	// fill in the combo
	buildSystemCommandsCombo();
	
	// select the first command to be active in the list
	std::string active = "";
	if( settings_->getCommand( 80, active ) )
	{
		cmbSystemCommands_.set_active_text( active );
	}
	else
	{
		cmbSystemCommands_.set_active_text( "80" );
	}
	
	// edit button
	Gtk::Button btnEditCommand( "Edit" );
	tblSystemCommands.attach( btnEditCommand, 1, 2, 1, 2,
		Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	btnEditCommand.signal_clicked().connect(
		sigc::mem_fun( *this, &Ui::showSystemCommandDialog ) );
	
	// clear button
	Gtk::Button btnClearCommand( "Clear" );
	tblSystemCommands.attach( btnClearCommand, 2, 3, 1, 2,
		Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	btnClearCommand.signal_clicked().connect(
		sigc::mem_fun( *this, &Ui::removeSystemCommand ) );
	
	
	// make all child widgets visible
	ntbkSettings_.show_all();
	
	// show the dialog
	int result = dlgSettings.run();
	
	switch( result )
	{
		// save settings
		case Gtk::RESPONSE_ACCEPT:
		{
			settings_->setFullscreenStatus( btnFullscreen.get_active() );
			settings_->setAutoconnectStatus( btnAutoConnect.get_active() );
			settings_->setUseDefaultProfile( btnDefaultProfile.get_active() );
			
			break;
		}
		// discard settings
		case Gtk::RESPONSE_CANCEL:
		case Gtk::RESPONSE_DELETE_EVENT:
		{
			break;
		}
		default:
		{
			break;
		}
	}
}

// (Re-)build profiles combo box in Settings dialog
void Ui::buildProfilesCombo()
{
	// clear the combo
	cmbProfiles_.clear_items();
	
	// number of profiles
	unsigned int profiles = settings_->getNumOfProfiles();
	
	// fill in the combo box with profile names
	for( unsigned int i = 0; i < profiles; ++i )
	{
		Profile profile = {};
		
		// get profile name
		if( settings_->getProfile( i, profile ) ) 
		{
			cmbProfiles_.append_text( profile.name );
		}
	}
}

// (Re-)build system commands combo box in Settings dialog
void Ui::buildSystemCommandsCombo()
{
	// clear the combo
	cmbSystemCommands_.clear_items();
	
	// fill in the combo box with command names
	for( unsigned int i = 0; i < NUMBER_OF_COMMANDS; ++i )
	{
		std::string commandName = "";
		if( settings_->getCommand( i + 80, commandName ) )
		{
			cmbSystemCommands_.append_text( commandName );
		}
		else
		{
			cmbSystemCommands_.append_text( Routines::int2string( i + 80 ) );
		}
	}
}

// Show dialog to create new Connection Profile
void Ui::showNewProfileDialog()
{
	#ifdef DEBUG
		std::cout << "at showNewProfileDialog()" << std::endl;
	#endif
	
	showProfileDialog( NULL );
}

// Show dialog to edit old Connection Profile
void Ui::showEditProfileDialog()
{
	#ifdef DEBUG
		std::cout << "at showEditProfileDialog()" << std::endl;
	#endif
	
	// get the profile using the name of selected profile
	Profile profile = {};	
	if( settings_->getProfile( cmbProfiles_.get_active_text(), profile ) )
	{
		showProfileDialog( &profile );
	}
}

// Show dialog to edit Connection Profile
void Ui::showProfileDialog( Profile* profile )
{
	#ifdef DEBUG
		std::cout << "at showProfileDialog()" << std::endl;
	#endif
	
	Gtk::Dialog* dlgProfile = NULL;
	
	if( profile == NULL )
	{
		dlgProfile = new Gtk::Dialog( "Create new profile", true, true );
	}
	else
	{
		dlgProfile = new Gtk::Dialog( "Edit profile", true, true );
	}
	
	// add buttons to the dialog
	dlgProfile->add_button( Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT );
	dlgProfile->add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
	
	Gtk::Table tblProfile( 3, 4, false );
	dlgProfile->get_vbox()->pack_start( tblProfile );
	
	// entry for profile name
	Gtk::Entry ntrName;
	tblProfile.attach( ntrName, 0, 2, 3, 4, Gtk::FILL, Gtk::FILL, 2, 4 );
	
	// connection method WLAN
	Gtk::RadioButton btnUseWlan( "WLAN" );
	tblProfile.attach( btnUseWlan, 0, 1, 4, 5 );
	
	Gtk::HBox hbxWlan( false, 4 );
	tblProfile.attach( 
		hbxWlan, 1, 4, 4, 5, Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	
	// IP title and text entry
	Gtk::Label lblIp( "IP" );
	//tblProfile.attach( lblIp, 1, 2, 4, 5 );
	hbxWlan.pack_start( lblIp, false, false, 0 );
	
	Gtk::Entry ntrIp;
	ntrIp.set_max_length( 15 );
	//tblProfile.attach( ntrIp, 2, 3, 4, 5 );
	hbxWlan.pack_start( ntrIp, true, true, 0 );
	
	// Port label and port selector
	Gtk::Label lblPort( "Port" );
	hbxWlan.pack_start( lblPort, false, false, 0 );
	
	Hildon::NumberEditor nbrPort( 1024, 65535 );
	nbrPort.set_range( 1024, 65535 );
	hbxWlan.pack_start( nbrPort, false, false, 0 );
	
	// connection method BT
	Gtk::RadioButtonGroup group = btnUseWlan.get_group();
	Gtk::RadioButton btnUseBt( group, "Bluetooth" );
	tblProfile.attach( btnUseBt, 0, 1, 5, 6 );
	
	Gtk::HBox hbxBluetooth( false, 4 );
	tblProfile.attach( 
		hbxBluetooth, 1, 4, 5, 6, Gtk::FILL | Gtk::EXPAND, Gtk::FILL, 2, 4 );
	
	// BDA title and text entry
	//Gtk::Label lblBda( "BDA" );
	//tblProfile.attach( lblBda, 1, 2, 5, 6 );
	//hbxBluetooth.pack_start( lblBda, false, false, 0 );
	
	ntrBda_ = new Gtk::Entry();
	ntrBda_->set_width_chars( 17 );
	hbxBluetooth.pack_start( *ntrBda_, true, true, 0 );
	
	// BT search button 
	Gtk::Button btnSearchBt( "Search" );
	hbxBluetooth.pack_start( btnSearchBt, false, false, 0 );
	btnSearchBt.signal_clicked().connect( 
		sigc::mem_fun( *this, &Ui::searchBtDevices ) );
	
	// check if editing old
	if( profile == NULL )
	{
		ntrName.set_text( "Profile name" );
		nbrPort.set_value( 6547 );
	}
	else
	{
		// set profile name
		ntrName.set_text( profile->name );
		
		// select right connection method
		if( profile->wlan )
		{
			btnUseWlan.set_active();
			ntrIp.set_text( profile->ip );
			nbrPort.set_value( profile->port );
		}
		else if( profile->bluetooth )
		{
			btnUseBt.set_active();
			ntrBda_->set_text( profile->bda );
			nbrPort.set_value( 6547 );
		}
	}
	
	tblProfile.show_all();
	
	// show the dialog
	int result = dlgProfile->run();
	
	switch( result )
	{
		// save settings
		case Gtk::RESPONSE_ACCEPT:
		{
			if( btnUseWlan.get_active() )
			{
				createProfile( ntrName.get_text(), true, ntrIp.get_text(), 
					nbrPort.get_value() );
			}
			else if( btnUseBt.get_active() )
			{
				createProfile( ntrName.get_text(), false, ntrBda_->get_text() );
			}
			
			// update combo box
			buildProfilesCombo();
			// select active the new/edited profile
			cmbProfiles_.set_active_text( ntrName.get_text() );
			// rebuild Select Profile -submenu
			buildSelectProfileSubmenu();
			
			break;
		}
		// discard settings
		case Gtk::RESPONSE_CANCEL:
		case Gtk::RESPONSE_DELETE_EVENT:
		{
			break;
		}
		default:
		{
			break;
		}
	}
	
	delete dlgProfile;
	dlgProfile = NULL;
	
	delete ntrBda_;
	ntrBda_ = NULL;
	
}

// Create a new Connection Profile to the application
bool Ui::createProfile( std::string name, bool isWlan, std::string address,
	int port )
{
	// check that name is given
	if( name == "" )
	{
		return false;
	}
	
	// create new profile according to user settings
	Profile profile = {};
	profile.name = name;
	
	if( isWlan )
	{
		profile.wlan = true;
		profile.ip = address;
		profile.bluetooth = false;
		profile.bda = "";
	}
	else
	{
		profile.wlan = false;
		profile.ip = "";
		profile.bluetooth = true;
		profile.bda = address;
	}
	
	profile.port = port;
	
	return settings_->addProfile( profile );
}

// Callback function for connection profile delete button
void Ui::deleteProfile()
{
	// get the selected profile name
	std::string name = cmbProfiles_.get_active_text();
	
	// remove profile from settings
	if( !settings_->deleteProfile( name ) )
	{
		showMessage( "Could not delete profile " + name );
	}
	
	// remove profile from combo box
	cmbProfiles_.remove_text( name );
	
	// set active the first profile
	Profile profile = {};
	
	if( settings_->getProfile( 0, profile ) )
	{
		cmbProfiles_.set_active_text( profile.name );
	}
	
	buildSelectProfileSubmenu();
}

// Callback function for connection profile set default button
void Ui::setDefaultProfile()
{
	// get the selected profile name and set it default
	settings_->setDefaultProfile( cmbProfiles_.get_active_text() );
	
	// construct select profile submenu again
	buildSelectProfileSubmenu();
	
}

// Show a dialog to edit system command menu titles
void Ui::showSystemCommandDialog()
{	
	Gtk::Dialog dlgSystemCommand( "Edit system command menu item", true, true );
	
	// add buttons to the dialog
	dlgSystemCommand.add_button( Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT );
	dlgSystemCommand.add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
	
	Gtk::HBox hBoxCommand;
	dlgSystemCommand.get_vbox()->pack_start( hBoxCommand );
	
	// show command id
	Gtk::Label lblCommand( "Command:" );
	hBoxCommand.pack_start( lblCommand );
	
	unsigned int number = 0;
	if( !settings_->getCommand( cmbSystemCommands_.get_active_text(), number ) )
	{
		// try to treat it as a number
		if( !Routines::string2int( cmbSystemCommands_.get_active_text(), number ) )
		{
			std::cout << "Error gettings commands id" << std::endl;
		}
	}
	
	if( number < 80 || number > 89 )
	{
		showMessage( "Error" );
		return;
	}
	
	std::string id = Routines::int2string( number );
	Gtk::Label lblId( id );
	hBoxCommand.pack_start( lblId );
	
	Gtk::HBox hBoxTitle;
	dlgSystemCommand.get_vbox()->pack_start( hBoxTitle );
	
	// show menu title
	Gtk::Label lblTitle( "Menu title:" );
	hBoxTitle.pack_start( lblTitle );
	
	// create text field for command
	Gtk::Entry ntrName;
	hBoxTitle.pack_start( ntrName );
	
	ntrName.set_text( cmbSystemCommands_.get_active_text() );
	
	dlgSystemCommand.show_all();
	
	// show the dialog
	int result = dlgSystemCommand.run();
	
	switch( result )
	{
		// save changes
		case Gtk::RESPONSE_ACCEPT:
		{
			std::string name = ntrName.get_text();
			updateSystemCommand( name, id );
			buildSystemCommandsCombo();
			
			if( name == "" )
			{
				std::string commandName = "";
				if( settings_->getCommand( 80, commandName ) )
				{
					cmbSystemCommands_.set_active_text( commandName );
				}
				else
				{
					cmbSystemCommands_.set_active_text( "80" );
				}
			}
			else
			{
				cmbSystemCommands_.set_active_text( name );
			}
			
			break;
		}
		// discard changes
		case Gtk::RESPONSE_CANCEL:
		case Gtk::RESPONSE_DELETE_EVENT:
		{
			break;
		}
		default:
		{
			break;
		}
	}
	
}

// Update system command information
bool Ui::updateSystemCommand( std::string const& name, std::string const& id )
{
	#ifdef DEBUG
		std::cout << "@updateSystemCommand" << std::endl;
	#endif
	
	// add command to settings
	settings_->saveCommand( id, name );
	
	// build the menu again
	buildSystemCommandsSubmenu();
	
	return true;
}

// Show application help to user
void Ui::showHelp()
{
	#ifndef OS2007
	osso_return_t retVal = hildon_help_show( osso_, 
		OSSO_HELP_INTRO.c_str(), 0 );
	#else
	osso_return_t retVal = ossohelp_show( osso_, 
		OSSO_HELP_INTRO.c_str(), 0 );
	#endif
	
	if( retVal == OSSO_ERROR )
	{
		showMessage( "Sorry, no help available in your language" );
	}
}

// Show application about dialog
void Ui::showAbout()
{
	#ifndef OS2007
	osso_return_t retVal = hildon_help_show( osso_, 
		OSSO_HELP_ABOUT.c_str(), HILDON_HELP_SHOW_DIALOG );
	#else
	osso_return_t retVal = ossohelp_show( osso_, 
		OSSO_HELP_ABOUT.c_str(), OSSO_HELP_SHOW_DIALOG );
	#endif
	
	if( retVal == OSSO_ERROR )
	{
		showMessage( "Sorry, About is not available in your language" );
	}
}

// Change application fullscreen status
void Ui::toggleFullscreen()
{
	if( !fullscreen_ )
	{
		// change to fullscreen
		Window::fullscreen();
		fullscreen_ = true;
	}
	else
	{
		// change to normal view
		Window::unfullscreen();
		fullscreen_ = false;
	}
}

// Callback function for hardware key presses
bool Ui::hwKeyPressed( GdkEventKey* event )
{
	if( !event )
	{
		return false;
	}
	
	// check which key was pressed
	switch( event->keyval )
	{
		case GDK_Up: // Move up
		{
			userAction( ACTION_UP );
			break;
		}
		case GDK_Down: // Move down
		{
			userAction( ACTION_DOWN );
			break;
		}
		case GDK_Left: // Move left
		{
			userAction( ACTION_LEFT );
			break;
		}
		case GDK_Right: // Move right
		{
			userAction( ACTION_RIGHT );
			break;
		}
		case GDK_Return: // Select, Confirm
		{
			userAction( ACTION_ENTER );
			break;
		}
		case GDK_Escape: // Cancel, Close
		{
			userAction( ACTION_ESC );
			break;
		}
		case GDK_F6: // Full screen
		{
			toggleFullscreen();
			break;
		}
		case GDK_F7: // Increase
		{
			userAction( ACTION_VOL_UP );
			break;
		}
		case GDK_F8: // Decrease
		{
			userAction( ACTION_VOL_DOWN );
			break;
		}
		default:
			return false;
		
	}
	
	// stop the event
	return true;
}

// Construct application main menu
void Ui::buildMenu()
{
	// Go to -submenu
	menu_.append( itemGoto_ );
	itemGoto_.set_submenu( gotoSubmenu_ );
	
	// item Main Menu
	gotoSubmenu_.append( itemMainMenu_ );
	itemMainMenu_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_MAIN_MENU ) );
		
	// item Program Guide
	gotoSubmenu_.append( itemProgramGuide_ );
	itemProgramGuide_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_PROGRAM_GUIDE ) );
	
	// item Live TV
	gotoSubmenu_.append( itemLiveTV_ );
	itemLiveTV_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_LIVE_TV ) );
	
	// item TV Recordings
	gotoSubmenu_.append( itemTVRecordings_ );
	itemTVRecordings_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_TV_RECORDINGS ) );
		
	// item MythVideo
	gotoSubmenu_.append( itemMythVideo_ );
	itemMythVideo_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_MYTHVIDEO ) ) ;
	
	// item Play DVD		
	gotoSubmenu_.append( itemPlayDVD_ );
	itemPlayDVD_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_PLAY_DVD ) );
	
	// item Play Music
	gotoSubmenu_.append( itemPlayMusic_ );
	itemPlayMusic_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_PLAY_MUSIC ) );
	
	
	// Extra commands -submenu
	menu_.append( itemExtraCommands_ );
	itemExtraCommands_.set_submenu( extraCommandsSubmenu_ );
	
	// item Aspect Ratio
	extraCommandsSubmenu_.append( itemAspectRatio_ );
	itemAspectRatio_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_ASPECT_RATIO ) );
	
	// item Time Stretch
	extraCommandsSubmenu_.append( itemTimeStretch_ );
	itemTimeStretch_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_TIME_STRETCH ) );
	
	// item Input Card
	extraCommandsSubmenu_.append( itemInputCard_ );
	itemInputCard_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_INPUT_CARD ) );
	
	// item Sleep Timer
	extraCommandsSubmenu_.append( itemSleepTimer_ );
	itemSleepTimer_.signal_activate().connect( 
		sigc::bind< std::string >( sigc::mem_fun( 
		*this, &Ui::userAction ), ACTION_SLEEP_TIMER ) );
		
	extraCommandsSubmenu_.append( itemSep0_ );
	
	// item Custom Command
	extraCommandsSubmenu_.append( itemCustomCommand_ );
	itemCustomCommand_.signal_activate().connect( 
		sigc::mem_fun( *this, &Ui::showCustomCommandDialog ) );
	
	// System commands -submenu
	menu_.append( itemSystemCommands_ );
	buildSystemCommandsSubmenu();
	
	// separator
	menu_.append( itemSep1_ );
	
	// Change connection -submenu
	menu_.append( itemSelectProfile_ );
	buildSelectProfileSubmenu();
	
	// separator
	menu_.append( itemSep4_ );
	
	// item "open connection"
	menu_.append( itemOpenConnection_ );
	itemOpenConnection_.signal_activate().connect( 
		sigc::mem_fun( *this, &Ui::openConnection ) );
	
	
	// item "close connection"
	menu_.append( itemCloseConnection_ );
	itemCloseConnection_.signal_activate().connect( 
		sigc::mem_fun( connection_, &Connection::closeConnection ) );
	
	// separator
	menu_.append( itemSep2_ );
	
	// Tools -submenu
	menu_.append( itemTools_ );
	itemTools_.set_submenu( toolsSubmenu_ );
	
	// item Settings
	toolsSubmenu_.append( itemSettings_ );
	itemSettings_.signal_activate().connect( 
		sigc::mem_fun( *this, &Ui::showSettingsDialog ) );
	
	// item Help
	toolsSubmenu_.append( itemHelp_ );
	itemHelp_.signal_activate().connect( 
		sigc::mem_fun( *this, &Ui::showHelp ) );
	
	// item About
	toolsSubmenu_.append( itemAbout_ );
	itemAbout_.signal_activate().connect( 
		sigc::mem_fun( *this, &Ui::showAbout ) );
	
	// separator
	menu_.append( itemSep3_ );
	
	// item "Close"
	menu_.append( itemClose_ );
	itemClose_.signal_activate().connect( 
		sigc::mem_fun( *this, &Window::hide ) );
	
	
	
	// show the menu and all submenus and items
	menu_.show_all();
	
	// set the menu used by application
	set_menu( menu_ );
	
}

// Construct System commands -submenu
void Ui::buildSystemCommandsSubmenu()
{
	#ifdef DEBUG
		std::cout << "@buildSystemCommandsSubmenu()" << std::endl;
	#endif
	
	if( systemCommandsSubmenu_ != NULL )
	{
		delete systemCommandsSubmenu_;
		systemCommandsSubmenu_ = NULL;
	}
	
	clearSystemCommands();
	
	systemCommandsSubmenu_ = new Gtk::Menu;
	
	bool empty = true;
	
	// fill in the menu with command names
	for( unsigned int i = 0; i < NUMBER_OF_COMMANDS; ++i )
	{
		std::string commandName = "";
		
		if( settings_->getCommand( i + 80, commandName ) )
		{
			if( commandName != "" )
			{
				empty = false;
				
				Command* command = new Command;
				command->name = commandName;
				
				std::string id = Routines::int2string( i + 80 );
				command->command = "04";
				command->command.append( id );
				
				// create a new menu item and save pointer
				Gtk::MenuItem* itemCommand = new Gtk::MenuItem( commandName );
				command->menuItem = itemCommand;
				
				systemCommands_.push_back( command );
	
				// append, attach command, and show the menu item
				systemCommandsSubmenu_->append( *itemCommand );
				itemCommand->signal_activate().connect( 
					sigc::bind< std::string >( sigc::mem_fun( 
					*this, &Ui::userAction ), command->command ) );
			}
			else
			{
				std::cout << "empty command name for command " 
				<< i << std::endl;
			}
		}
	}
	
	if( empty )
	{
		Command* command = new Command;
		
		// create a new menu item and save pointer
		Gtk::MenuItem* itemCommand = new Gtk::MenuItem( "Edit system commands" );
		command->menuItem = itemCommand;
		
		systemCommands_.push_back( command );

		// append, attach command, and show the menu item
		systemCommandsSubmenu_->append( *itemCommand );
		itemCommand->signal_activate().connect( 
			sigc::mem_fun( *this, &Ui::showSettingsDialog ) );
	}
		
	itemSystemCommands_.set_submenu( *systemCommandsSubmenu_ );
	itemSystemCommands_.show_all();
	
}

// Remove system command from application
void Ui::removeSystemCommand()
{
	unsigned int number = 0;
	if( !settings_->getCommand( cmbSystemCommands_.get_active_text(), number ) )
	{
		std::cout << "already number, exiting" << std::endl;
	}
	
	std::string id = Routines::int2string( number );
	
	updateSystemCommand( "", id );
	buildSystemCommandsCombo();
	
	cmbSystemCommands_.set_active_text( id );
}

// Clear all system command menu items
void Ui::clearSystemCommands()
{
	#ifdef DEBUG
		std::cout << "@clearSystemCommands()" << std::endl;
	#endif
	
	// release mem and clear all system command items
	for( unsigned int i = 0; i < systemCommands_.size(); ++i )
	{
		delete systemCommands_.at( i )->menuItem;
		systemCommands_.at( i )->menuItem = 0;
	}
	systemCommands_.clear();
}

// Construct Connection profile -submenu
void Ui::buildSelectProfileSubmenu()
{
	buildingSubmenu_ = true;
	
	#ifdef DEBUG
		std::cout << "@buildSelectProfileSubmenu()" << std::endl;
	#endif
	
	if( selectProfileSubmenu_ != NULL )
	{
		delete selectProfileSubmenu_;
		selectProfileSubmenu_ = 0;
	}
	
	// clear old items
	clearSelectProfileMenuItems();
	
	selectProfileSubmenu_ = new Gtk::Menu;
	
	// create group for menu items
	Gtk::RadioMenuItem::Group profileGroup;
	
	bool empty = true;
	
	// get active profile
	Profile current = {};
	if( !settings_->getActiveProfile( current ) )
	{
		std::cout << "Could not get active profile settings" << std::endl;
	}
		
	// fill in the menu with profile names
	unsigned int profiles = settings_->getNumOfProfiles();
	
	for( unsigned int i = 0; i < profiles; ++i )
	{
		Profile profile = {};
		
		if( settings_->getProfile( i, profile ) )
		{
			if( profile.name != "" )
			{
				empty = false;
				
				// create a new menu item and save pointer
				Gtk::RadioMenuItem* itemProfile = new Gtk::RadioMenuItem( 
					profileGroup, profile.name );
				
				// put active profile selected
				if( profile.name == current.name )
				{
					itemProfile->set_active();
					settings_->setActiveProfile( profile.name );
					connection_->changeConnectionProfile( profile.name );
				}
				
				profileMenuItems_.push_back( itemProfile );
	
				// append, attach command, and show the menu item
				selectProfileSubmenu_->append( *itemProfile );
				//itemProfile->signal_activate().connect(
				//	sigc::mem_fun( *this, &Ui::changeConnection ) );
				
				itemProfile->signal_activate().connect( 
					sigc::bind< std::string >( sigc::mem_fun( 
					*this, &Ui::changeConnection ), profile.name ) );
			}
			
		}
	}
	
	if( empty )
	{
		// create a new menu item and save pointer
		Gtk::MenuItem* itemProfile = new Gtk::MenuItem( "Edit profiles" );
		
		profileMenuItems_.push_back( itemProfile );

		// append, attach command, and show the menu item
		selectProfileSubmenu_->append( *itemProfile );
		itemProfile->signal_activate().connect( 
			sigc::mem_fun( *this, &Ui::showSettingsDialog ) );
	}
	
	itemSelectProfile_.set_submenu( *selectProfileSubmenu_ );
	itemSelectProfile_.show_all();
	
	buildingSubmenu_ = false;
}

// Clear all Connection profile menu items
void Ui::clearSelectProfileMenuItems()
{
	#ifdef DEBUG
		std::cout << "@clearSelectProfileMenuItems()" << std::endl;
	#endif
	
	// release mem and clear all menu items
	for( unsigned int i = 0; i < profileMenuItems_.size(); ++i )
	{
		delete profileMenuItems_.at( i );
		//profileMenuItems_.at( i ) = 0;
	}
	
	profileMenuItems_.clear();
}

// Callback function for changing active connection
void Ui::changeConnection( std::string const& name )
{
	if( buildingSubmenu_ )
	{
		return;
	}
	
	#ifdef DEBUG
		std::cout << "@changeConnection()" << std::endl;
	#endif
	
	Profile profile = {};
	settings_->getActiveProfile( profile );
	
	if( name != profile.name )
	{
		if( connection_->isConnected() )
		{
			std::string msg( "You are about to disconnect from " );
			msg.append( profile.name );
			msg.append( " and establish a connection to " );
			msg.append( name );
			msg.append( ". Proceed?" );
			
			Hildon::Note note( Hildon::NOTE_TYPE_CONFIRMATION_BUTTON, msg );
			note.add_button( Gtk::Stock::YES, Gtk::RESPONSE_YES );
			note.add_button( Gtk::Stock::NO, Gtk::RESPONSE_NO );
			
			int result = note.run();
			
			switch( result )
			{
				case Gtk::RESPONSE_YES:
				{
					break;
				}
				case Gtk::RESPONSE_NO:
				default:
				{
					buildSelectProfileSubmenu();
					return;	
				}
			}
			
		}
		
		bool connected = connection_->isConnected();
		
		if( !settings_->setActiveProfile( name ) || 
			!connection_->changeConnectionProfile( name ) )
		{
			showMessage( "Changing connection profile failed" );
			return;
		}
		
		if( connected )
		{
			openConnection();	
		}
	}
}

// Construct the main User Interface
void Ui::buildInterface()
{
	add( hBox_ );
	
	// devide the main pane to three parts
	
	// build left pane
	vBoxLeft_.set_border_width( 3 );
	hBox_.pack_start( vBoxLeft_, false, false, 0 );
	
	// button up
	btnUp_.set_size_request( SMALL_BUTTON_SIZE, SMALL_BUTTON_SIZE );
	btnUp_.set_image( imgUp_ );
	vBoxLeft_.pack_start( btnUp_, false, false, 0 );
	btnUp_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_UP ) );
	
	// button enter
	btnEnter_.set_image( imgEnter_ );
	vBoxLeft_.pack_start( btnEnter_, true, true, 0 );
	btnEnter_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_ENTER ) );
	
	// button down
	btnDown_.set_size_request( SMALL_BUTTON_SIZE, SMALL_BUTTON_SIZE );
	btnDown_.set_image( imgDown_ );
	vBoxLeft_.pack_start( btnDown_, false, false, 0 );
	btnDown_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_DOWN ) );
	
	
	// build center pane
	vBoxCenter_.set_border_width( 3 );
	hBox_.pack_start( vBoxCenter_, true, true, 0 );
	
	// first row
	hBox1_.set_border_width( 3 );
	vBoxCenter_.pack_start( hBox1_, true, true, 0 );
	
	// button volume up
	btnVolUp_.set_size_request( NUMBER_BUTTON_WIDTH, NUMBER_BUTTON_HEIGHT );
	btnVolUp_.set_image( imgVolUp_ );
	hBox1_.pack_start( btnVolUp_, false, false, 0 );
	btnVolUp_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_VOL_UP ) );
	
	// button 1
	hBox1_.pack_start( btnOne_, false, true, 0 );
	btnOne_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_ONE ) );
		
	// button 2
	hBox1_.pack_start( btnTwo_, false, true, 0 );
	btnTwo_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_TWO ) );
	
	// button 3
	hBox1_.pack_start( btnThree_, false, true, 0 );
	btnThree_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_THREE ) );
	
	// button channel up
	btnChUp_.set_size_request( NUMBER_BUTTON_WIDTH, NUMBER_BUTTON_HEIGHT );
	btnChUp_.set_image( imgChUp_ );
	hBox1_.pack_start( btnChUp_, false, false, 0 );
	btnChUp_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_CH_UP ) );
	
	// second row
	hBox2_.set_border_width( 3 );
	vBoxCenter_.pack_start( hBox2_, true, true, 0 );
	
	// label Volume
	hBox2_.pack_start( lblVol_, false, true, 0 );
	
	// button 4
	hBox2_.pack_start( btnFour_, false, true, 0 );
	btnFour_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_FOUR ) );
	
	// button 5
	hBox2_.pack_start( btnFive_, false, true, 0 );
	btnFive_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_FIVE ) );
	
	// button 6
	hBox2_.pack_start( btnSix_, false, true, 0 );
	btnSix_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_SIX ) );
	
	// label Channel
	hBox2_.pack_start( lblCh_, false, true, 0 );
	
	// third row
	hBox3_.set_border_width( 3 );
	vBoxCenter_.pack_start( hBox3_, true, true, 0 );
	
	// button volume down
	btnVolDown_.set_size_request( NUMBER_BUTTON_WIDTH, NUMBER_BUTTON_HEIGHT );
	btnVolDown_.set_image( imgVolDown_ );
	hBox3_.pack_start( btnVolDown_, false, false, 0 );
	btnVolDown_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_VOL_DOWN ) );
		
	// button 7
	hBox3_.pack_start( btnSeven_, false, true, 0 );
	btnSeven_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_SEVEN ) );
	
	// button 8
	hBox3_.pack_start( btnEight_, false, true, 0 );
	btnEight_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_EIGHT ) );
		
	// button 9
	hBox3_.pack_start( btnNine_, false, true, 0 );
	btnNine_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_NINE ) );
	
	// button channel down
	btnChDown_.set_size_request( NUMBER_BUTTON_WIDTH, NUMBER_BUTTON_HEIGHT );
	btnChDown_.set_image( imgChDown_ );
	hBox3_.pack_start( btnChDown_, false, false, 0 );
	btnChDown_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_CH_DOWN ) );
	
	// fourth row
	hBox4_.set_border_width( 3 );
	vBoxCenter_.pack_start( hBox4_, true, true, 0 );
	
	// button menu
	hBox4_.pack_start( btnMenu_, false, true, 0 );
	btnMenu_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_MENU ) );
	
	// empty
	hBox4_.pack_start( lblEmpty1_, false, true, 0 );
	
	// button 0
	hBox4_.pack_start( btnZero_, false, true, 0 );
	btnZero_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_ZERO ) );
	
	// empty
	hBox4_.pack_start( lblEmpty2_, false, true, 0 );
	
	// button Esc
	hBox4_.pack_start( btnEsc_, false, true, 0 );
	btnEsc_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_ESC ) );
		
	// fifth row
	hBox5_.set_border_width( 3 );
	vBoxCenter_.pack_start( hBox5_, true, true, 0 );
	
	// button rewind
	btnRewind_.set_image( imgRewind_ );
	hBox5_.pack_start( btnRewind_, false, true, 0 );
	btnRewind_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_REWIND ) );
		
	// button pause
	btnPause_.set_image( imgPause_ );
	hBox5_.pack_start( btnPause_, false, true, 0 );
	btnPause_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_PAUSE ) );
	
	// button forward
	btnForward_.set_image( imgForward_ );
	hBox5_.pack_start( btnForward_, false, true, 0 );
	btnForward_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_FORWARD ) );
		
	// sixth row
	hBox6_.set_border_width( 3 );
	vBoxCenter_.pack_start( hBox6_, true, true, 0 );
	
	// button previous
	btnPrevious_.set_image( imgPrevious_ );
	hBox6_.pack_start( btnPrevious_, false, true, 0 );
	btnPrevious_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_PREVIOUS ) );
	
	// button record
	btnRecord_.set_image( imgRecord_ );
	hBox6_.pack_start( btnRecord_, false, true, 0 );
	btnRecord_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_RECORD ) );
	
	// button next
	btnNext_.set_image( imgNext_ );
	hBox6_.pack_start( btnNext_, false, true, 0 );
	btnNext_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_NEXT ) );
	
	
	// build right pane	
	vBoxRight_.set_border_width( 3 );
	hBox_.pack_start( vBoxRight_, false, true, 0 );
	
	// button right
	btnRight_.set_size_request( SMALL_BUTTON_SIZE, SMALL_BUTTON_SIZE );
	btnRight_.set_image( imgRight_ );
	vBoxRight_.pack_start( btnRight_, false, false, 0 );
	btnRight_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_RIGHT ) );
	
	// button enter
	btnEnterRight_.set_image( imgEnterRight_ );
	vBoxRight_.pack_start( btnEnterRight_, true, true, 0 );
	btnEnterRight_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_ENTER ) );
	
	// button left
	btnLeft_.set_size_request( SMALL_BUTTON_SIZE, SMALL_BUTTON_SIZE );
	btnLeft_.set_image( imgLeft_ );
	vBoxRight_.pack_start( btnLeft_, false, false, 0 );
	btnLeft_.signal_clicked().connect( sigc::bind< std::string >(
		sigc::mem_fun( *this, &Ui::userAction ), ACTION_LEFT ) );
	
}

// Start searching Bluetooth devices
void Ui::searchBtDevices()
{
	DBusError error;
	dbus_error_init( &error );
	
	if( !dbus_bus_name_has_owner( dbus_, CONBTDIALOGS_DBUS_SERVICE, &error ) )
	{
		std::cout << "Name has no owner on the bus" << std::endl;
		return;
	}
	
	DBusMessage* message = dbus_message_new_method_call( 
		CONBTDIALOGS_DBUS_SERVICE, CONBTDIALOGS_DBUS_PATH, 
		CONBTDIALOGS_DBUS_INTERFACE, CONBTDIALOGS_SEARCH_REQ );
	
	if( message == NULL )
	{
		std::cout << "Could not form dbus message" << std::endl;
		return;
	}
	
	const char* majorClass = "computer";
	const char* minorClass = "";
	const char array[] = {};
	const char* parray = array;
	const char* bonding = "";
	
	if( !dbus_message_append_args( message, 
		DBUS_TYPE_STRING, &majorClass, DBUS_TYPE_STRING, &minorClass,
		DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &parray, 0, DBUS_TYPE_STRING, &bonding, DBUS_TYPE_INVALID ) )
	{
		std::cout << "Could not append arguments to dbus message: " 
		<< error.message << std::endl;
	}
	
	if( !dbus_connection_add_filter( dbus_, 
		Routines::btSearchHandler, this, NULL ) )
	{
		std::cout << "Could not set handler for bt search" << std::endl;
		return;
	}
	
	std::string filter( "type='signal',interface='" );
	filter += CONBTDIALOGS_DBUS_INTERFACE;
	filter += "'";
	
	dbus_bus_add_match( dbus_, filter.c_str(), NULL );
	
	dbus_error_init( &error );
	DBusMessage* reply = NULL;
	reply = dbus_connection_send_with_reply_and_block( dbus_, message, -1, &error );
	if( reply == NULL )
	{
		std::cout << "Could not send message via dbus: " << error.message << std::endl;
		return;
	}
	
	dbus_connection_flush( dbus_ );
	
	dbus_message_unref( message );
	message = NULL;
}
